
///////////////////////////////////////////////////////////////////////////////
//
//	image.cpp
//

#include "project.h"
#include "id.h"



ImageManager::ImageManager(ProjectManager *_project)
: project(_project)
{
	static_count		= GetPrivateProfileInt("StaticImage", "Count", 3432, RomInfoFileName);
	pStaticImageAdrs	= GetPrivateProfileInt("StaticImage", "ImageAddress", 0x3C8000, RomInfoFileName);
	pStaticActionAdrs	= GetPrivateProfileInt("StaticImage", "ActionAddress", 0x3CB600, RomInfoFileName);
	dynamic_count		= GetPrivateProfileInt("DynamicImage", "Count", 780, RomInfoFileName);
	pDynamicImageAdrs	= GetPrivateProfileInt("DynamicImage", "ImageAddress", 0x390000, RomInfoFileName);
	palette_count		= GetPrivateProfileInt("SpritePalettes", "Count", 234, RomInfoFileName);
	pPaletteAdrs		= GetPrivateProfileInt("SpritePalettes", "Address", 0x3D5FEE, RomInfoFileName);
}


ImageManager::~ImageManager(){
}


int ImageManager::get_static_count(){
	return static_count;
}

BANK ImageManager::get_static_action_bank(){
	return pStaticActionAdrs.bank;
}

LADRS ImageManager::get_static_image(int static_num){
	LADRS adrs = project->read_ladrs(pStaticImageAdrs.bank, pStaticImageAdrs.adrs + static_num);
	if (adrs.bank >= 0xC0)
	{
		adrs.bank -= 0xC0;
	}
	return adrs;
}

SADRS ImageManager::get_static_action(int static_num){
	return project->read_sadrs(pStaticActionAdrs.bank, pStaticActionAdrs.adrs + static_num/2);
}

int ImageManager::get_dynamic_count(){
	return dynamic_count;
}

BANK ImageManager::get_dynamic_bank(){
	return pDynamicImageAdrs.bank;
}

SADRS ImageManager::get_dynamic_image(int dynamic_num){
	return project->read_sadrs(pDynamicImageAdrs.bank, pDynamicImageAdrs.adrs + dynamic_num*4);
}

WORD ImageManager::get_dynamic_param(int dynamic_num){
	return project->read_word(pDynamicImageAdrs.bank, pDynamicImageAdrs.adrs + dynamic_num*4 + 2);
}

DynamicImageIterator ImageManager::get_dynamic(int dynamic_num){
	return DynamicImageIterator(this, get_dynamic_image(dynamic_num));
}


int ImageManager::get_palette_count(){
	return palette_count;
}

BANK ImageManager::get_palette_bank(){
	return pPaletteAdrs.bank;
}

SADRS ImageManager::get_palette_data(int palette_num){
	return project->read_sadrs(pPaletteAdrs.bank, pPaletteAdrs.adrs + palette_num*2);
}

WORD *ImageManager::get_palette(int palette_num){
	return (WORD*)project->get_pointer(pPaletteAdrs.bank, get_palette_data(palette_num));
}




//=================================================================================================

DynamicImageIterator::DynamicImageIterator(ProjectManager *_project, BANK _bank, SADRS _adrs)
: project(_project), bank(_bank), adrs(_adrs)
{
}

DynamicImageIterator::DynamicImageIterator(ImageManager *_image, SADRS _adrs)
: project(_image->get_project()), bank(_image->get_dynamic_bank()), adrs(_adrs)
{
}

DynamicImageIterator::DynamicImageIterator(const DynamicImageIterator &_src)
: project(_src.project), bank(_src.bank), adrs(_src.adrs)
{
}

DynamicImageIterator::~DynamicImageIterator(){
}

SADRS DynamicImageIterator::get_adrs() const{
	return adrs;
}

BYTE DynamicImageIterator::get_command() const{
	return project->read_byte(bank, adrs);
}

WORD DynamicImageIterator::get_value() const{
	return project->read_word(bank, adrs + 1);
}

WORD DynamicImageIterator::get_value2() const{
	return project->read_word(bank, adrs + 3);
}

void DynamicImageIterator::set_command(BYTE command) const{
	project->write_byte(command, bank, adrs);
}

void DynamicImageIterator::set_value(WORD value) const{
	project->write_word(value, bank, adrs + 1);
}

void DynamicImageIterator::set_value2(WORD value2) const{
	project->write_word(value2, bank, adrs + 3);
}

DynamicImageIterator DynamicImageIterator::operator ++(int dummy){
	DynamicImageIterator r = *this;
	++(*this);
	return r;
}

DynamicImageIterator &DynamicImageIterator::operator ++(){
	BYTE command = get_command();
	switch (command){
		case 0x90:		adrs += 5; break;
		default:		adrs += 3; break;
	}
	return *this;
}

DynamicImageIterator::operator bool(){
	return get_command() != 0x80;
}

bool DynamicImageIterator::operator ==(const DynamicImageIterator &r) const{
	return adrs == r.adrs;
}

bool DynamicImageIterator::operator !=(const DynamicImageIterator &r) const{
	return adrs != r.adrs;
}



//=================================================================================================


void StaticImageHeader8::load(ProjectManager *project, LADRS adrs){
	big_chip_count		= project->read_byte(adrs.bank, adrs.adrs);
	small_chip_count	= project->read_byte(adrs.bank, adrs.adrs + 1);
	small_chip_offset	= project->read_byte(adrs.bank, adrs.adrs + 2);
	third_chip_count	= project->read_byte(adrs.bank, adrs.adrs + 3);
	third_chip_offset	= project->read_byte(adrs.bank, adrs.adrs + 4);
	big2_chip_offset	= project->read_byte(adrs.bank, adrs.adrs + 5);
	b6					= project->read_byte(adrs.bank, adrs.adrs + 6);
	big2_chip_count2	= project->read_byte(adrs.bank, adrs.adrs + 7);
}

int StaticImageHeader8::get_all_chip_count(){
	return big_chip_count + small_chip_count + third_chip_count;
}

int StaticImageHeader8::get_pattern_small(int small_chip_num){
	return 8 + get_all_chip_count()*2 + small_chip_offset*32 + small_chip_num*32;
}

int StaticImageHeader8::get_pattern_big0(int big_chip_num){
	/*int big1_chip_count = big_chip_count - big2_chip_count2/2;
	return big_chip_num < big1_chip_count?
		8 + get_all_chip_count()*2 + big_chip_num*64 :
		8 + get_all_chip_count()*2 + big1_chip_count*128 + (big_chip_num - big1_chip_count)*64;*/
	return 8 + get_all_chip_count()*2 + (big_chip_num%8)*64 + (big_chip_num/8)*1024;
}
int StaticImageHeader8::get_pattern_big1(int big_chip_num){
	/*int big1_chip_count = big_chip_count - big2_chip_count2/2;
	return big_chip_num < big1_chip_count?
		8 + get_all_chip_count()*2 + big_chip_num*64 + 32 :
		8 + get_all_chip_count()*2 + big1_chip_count*128 + (big_chip_num - big1_chip_count)*64 + 32;*/
	return 8 + get_all_chip_count()*2 + (big_chip_num%8)*64 + (big_chip_num/8)*1024 + 32;
}
int StaticImageHeader8::get_pattern_big2(int big_chip_num){
	int big1_chip_count = big_chip_count - big2_chip_count2/2;
	return big_chip_num < big1_chip_count?
		8 + get_all_chip_count()*2 + (big_chip_num%8)*64 + (big_chip_num/8)*1024 + 512 :
		8 + get_all_chip_count()*2 + big2_chip_offset*32 + (big_chip_num - big1_chip_count)*64;
}
int StaticImageHeader8::get_pattern_big3(int big_chip_num){
	int big1_chip_count = big_chip_count - big2_chip_count2/2;
	return big_chip_num < big1_chip_count?
		8 + get_all_chip_count()*2 + (big_chip_num%8)*64 + (big_chip_num/8)*1024 + 32 + 512 :
		8 + get_all_chip_count()*2 + big2_chip_offset*32 + (big_chip_num - big1_chip_count)*64 + 32;
}

int StaticImageHeader8::get_pattern_third(int third_chip_num){
	return 8 + get_all_chip_count()*2 + third_chip_offset*32 + third_chip_num*32;
}







